<?php

namespace app\admin\controller\coupon;

use app\admin\service\coupon\CouponDetailService;
use app\admin\validate\coupon\CouponValidate;
use app\admin\validate\device\DeviceValidate;
use app\controller\Curd;
use app\model\Coupon;
use app\model\CouponDetail;
use app\model\CouponGoods;
use app\model\CouponPacket;
use app\model\Device;
use app\model\Member;
use app\model\SysDept;
use app\model\SysSerial;
use support\Db;
use support\exception\BusinessException;
use support\Request;
use support\Response;
use Tinywan\Jwt\JwtToken;

class CouponPacketController extends Curd
{
    public function __construct()
    {
        $this->model = new CouponPacket();
    }

    /**
     * @Desc 列表
     * @Author Gorden
     * @Date 2024/3/28 15:01
     *
     * @param Request $request
     * @return Response
     * @throws \support\exception\BusinessException
     */
    public function select(Request $request): Response
    {
        [$where, $format, $limit, $field, $order] = $this->selectInput($request);
        if (!empty($where['coupon_packet_addtimes'])) {
            $where['coupon_packet_addtimes'][0] = strtotime($where['coupon_packet_addtimes'][0]);
            $where['coupon_packet_addtimes'][1] = strtotime($where['coupon_packet_addtimes'][1]);
        }
        $order = $request->get('order', 'desc');
        $field = $field ?? 'coupon_packet_addtimes';
        $query = $this->doSelect($where, $field, $order);
        return $this->doFormat($query, $format, $limit);
    }

    protected function doSelect(array $where, string $field = null, string $order = 'desc')
    {
        $model = $this->model->with([
            'category' => function ($query) {
                $query->select('category_id', 'category_name');
            },
            'user',
            'dept'
//            'detail'
        ]);
        foreach ($where as $column => $value) {
            if (is_array($value)) {
                if ($value[0] === 'like' || $value[0] === 'not like') {
                    $model = $model->where($column, $value[0], "%$value[1]%");
                } elseif (in_array($value[0], ['>', '=', '<', '<>'])) {
                    $model = $model->where($column, $value[0], $value[1]);
                } elseif ($value[0] == 'in' && !empty($value[1])) {
                    $valArr = $value[1];
                    if (is_string($value[1])) {
                        $valArr = explode(",", trim($value[1]));
                    }
                    $model = $model->whereIn($column, $valArr);
                } elseif ($value[0] == 'not in' && !empty($value[1])) {
                    $valArr = $value[1];
                    if (is_string($value[1])) {
                        $valArr = explode(",", trim($value[1]));
                    }
                    $model = $model->whereNotIn($column, $valArr);
                } elseif ($value[0] == 'null') {
                    $model = $model->whereNull($column);
                } elseif ($value[0] == 'not null') {
                    $model = $model->whereNotNull($column);
                } elseif ($value[0] !== '' || $value[1] !== '') {
                    $model = $model->whereBetween($column, $value);
                }
            } else {
                $model = $model->where($column, $value);
            }
        }
        if ($field) {
            $model = $model->orderBy($field, $order);
        }
        return $model;
    }

    protected function afterQuery($items)
    {
        foreach ($items as &$item) {
            if (empty($item['dept'])) {
                $item['dept_all'] = [
                    'dept_id' => 0,
                    'dept_name' => "所有门店"
                ];
            }
        }

        return $items;
    }

    public function selectList()
    {
        $coupons = Coupon::where('coupon_status', 'ACTIVED')
            ->select('coupon_id', 'coupon_name')
            ->orderBy('coupon_addtimes', 'DESC')
            ->get()
            ->toArray();

        return json_success('', $coupons);
    }

//    public function afterQuery($items)
//    {
//        foreach ($items as &$item) {
//            $item->coupon_begindate = date('Y-m-d H:i:s', $item->coupon_begindate);
//            $item->coupon_enddate = date('Y-m-d H:i:s', $item->coupon_enddate);
//        }
//
//        return $items;
//    }

    /**
     * @Desc 添加优惠券
     * @Author Gorden
     * @Date 2024/8/14 15:16
     *
     * @param Request $request
     * @return Response
     */
    public function insert(Request $request): Response
    {
        if ($this->validate && !$this->validateClass->scene('add')->check($request->post())) {
            return json_fail($this->validateClass->getError());
        }
        Db::beginTransaction();
        try {
            $data = $this->insertInput($request);
            $data['join_coupon_packet_user_id'] = JwtToken::getCurrentId();
            $data['coupon_packet_status'] = 'PENDING';
            $this->doInsert($data);
            Db::commit();

            return json_success('success');
        } catch (BusinessException $customException) {
            Db::rollBack();
            dump($customException->getMessage());
            return json_fail($customException->getMessage());
        } catch (\Exception $e) {
            Db::rollBack();
            dump($e->getMessage());
            return json_fail('数据写入失败');
        }
    }

    /**
     * @Desc 关联优惠券、产品、规格
     * @Author Gorden
     * @Date 2024/8/14 14:32
     *
     * @param $goods
     * @param $couponId
     * @return void
     * @throws BusinessException
     */
    public function insertCouponGoods($goods, $couponId)
    {
        try {
            foreach ($goods as $good) {
                CouponGoods::insert([
                    'join_goods_coupon_id' => $couponId,
                    'join_coupon_goods_id' => $good['goods_id'],
                    'join_coupon_goods_sku_id' => $good['sku_id'],
                    'coupon_goods_json' => '[]',
                    'coupon_goods_extend_json' => '[]',
                    'coupon_goods_addtimes' => time()
                ]);
            }
        } catch (\Exception $e) {
            throw new BusinessException("关联产品失败");
        }
    }

    protected function insertInput(Request $request): array
    {
        $params = $request->post();
        $data = $this->inputFilter($params);
        $data['coupon_packet_id'] = "CP" . date("ymdH") . random_string(6, 'up');
        $coupons = [];
        if (!empty($params['coupons'])) {
            foreach ($params['coupons'] as $coupon) {
                $coupons[$coupon['id']]['num'] = $coupon['nbr'];
            }
        }

        $data['coupon_packet_json'] = json_encode(['coupons' => $coupons]);

        dump($data);
        return $data;
    }

    /**
     * @Desc 修改优惠券
     * @Author Gorden
     * @Date 2024/8/14 15:48
     *
     * @param Request $request
     * @return Response
     */
    public function update(Request $request): Response
    {
        Db::beginTransaction();
        try {
            [$id, $data] = $this->updateInput($request);

            $this->doUpdate($id, $data);

            Db::commit();

            return json_success('success');
        } catch (BusinessException $e) {
            Db::rollBack();
            return json_fail($e->getMessage());
        } catch (\Exception $e) {
            Db::rollBack();
            dump($e->getMessage());
            return json_fail('数据更新失败');
        }

    }

    protected function updateInput(Request $request): array
    {
        $params = $request->post();
        $primary_key = $this->model->getKeyName();
        $id = $request->post($primary_key);
        $data = $this->inputFilter($params);
        $model = $this->model->find($id);
        $coupons = [];
        if (!empty($params['coupons'])) {
            foreach ($params['coupons'] as $coupon) {
                $coupons[$coupon['id']]['num'] = $coupon['nbr'];
            }
        }
        $packetJson = [];
        if (!empty($model->coupon_packet_json)) {
            $packetJson = json_decode($model->coupon_packet_json, true);
        }
        $packetJson['coupons'] = $coupons;

        $data['coupon_packet_json'] = json_encode($packetJson);

        unset($data[$primary_key]);
        return [$id, $data];
    }

    /**
     * 优惠券详情
     */
    public function info(Request $request): Response
    {
        $packetId = $request->get('packet_id', '');
        if (!$packetId) {
            return json_fail('参数异常');
        }
        $packet = CouponPacket::where('coupon_packet_id', $packetId)->first();
        if (!$packet) {
            return json_fail('数据异常');
        }
        $coupons = [];
        if (!empty($packet->coupon_packet_json)) {
            $packetJson = json_decode($packet->coupon_packet_json, true);
            if (isset($packetJson['coupons'])) {
                $couponIds = array_keys($packetJson['coupons']);
                $couponModel = Coupon::whereIn('coupon_id', $couponIds)
                    ->select('coupon_id', 'coupon_name')
                    ->get()
                    ->toArray();
                foreach ($couponModel as $coupon) {
                    foreach ($packetJson['coupons'] as $key => $jsonCoupon) {
                        if ($coupon['coupon_id'] == $key) {
                            $coupons[] = [
                                'id' => $key,
                                'name' => $coupon['coupon_name'],
                                'nbr' => $jsonCoupon['num']
                            ];
                        }
                    }
                }
            }
        }

        $data = [
            'coupons' => $coupons
        ];

        return json_success('', $data);
    }

    /**
     * 删除
     * @param Request $request
     * @return Response
     * @throws BusinessException
     */
    public function delete(Request $request): Response
    {
        $ids = $this->deleteInput($request);
        Db::beginTransaction();
        try {
            if (CouponPacket::whereIn('coupon_packet_id', $ids)->where('coupon_packet_status', '=','ACTIVED')->exists()) {
                throw new BusinessException("有已发行的券包，无法删除");
            }
            $this->doDelete($ids);
            Db::commit();
            return json_success('success');
        } catch (BusinessException $e) {
            Db::rollBack();
            return json_fail($e->getMessage());
        } catch (\Exception $e) {
            dump($e->getMessage());
            Db::rollBack();
            return json_fail('删除失败');
        }
    }

    private function generateCouponDetail($couponId, $couponStatus, $nbr)
    {
        for ($i = 0; $i < intval($nbr); $i++) {
            CouponDetail::insert([
                'coupon_detail_id' => 'CUDT' . date("ymdH") . random_string(6, 'up'),
                'join_detail_coupon_id' => $couponId,
                'coupon_detail_status' => $couponStatus == 'ACTIVED' ? 'PENDING' : 'INIT',
                'coupon_detail_addtimes' => time()
            ]);
        }
    }
}